<?php
/*
Plugin Name: Gravity Forms CapsuleCRM Add-On
Plugin URI: http://www.mattersolutions.com.au
Description: Integrates Gravity Forms with CapsuleCRM allowing form submissions to be automatically sent to your CapsuleCRM account
Version: 0.9
Author: Matter Solutions
Author URI: http://www.mattersolutions.com.au

------------------------------------------------------------------------
Copyright 2012 Matter Solutions

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

add_action('init',  array('GFCapsuleCRM', 'init'));
register_activation_hook( __FILE__, array("GFCapsuleCRM", "add_permissions"));

ini_set('include_path',ini_get('include_path').':'.dirname(__FILE__).'/api/:');

require_once 'api/Services/Capsule.php';

class GFCapsuleCRM {

    private static $path = "gravityformscapsulecrm/capsulecrm.php";
    private static $url = "http://www.mattersolutions.com.au (URL)";
    private static $slug = "gravityformscapsulecrm";
    private static $version = "0.9";
    private static $min_gravityforms_version = "1.5";

    //Plugin starting point. Will load appropriate files
    public static function init(){

        if(RG_CURRENT_PAGE == "plugins.php"){
            //loading translations
            load_plugin_textdomain('gravityformscapsulecrm', FALSE, '/gravityformscapsulecrm/languages' );

            add_action('after_plugin_row_' . self::$path, array('GFCapsuleCRM', 'plugin_row') );

           //force new remote request for version info on the plugin page
            self::flush_version_info();
        }

        if(!self::is_gravityforms_supported()){
           return;
        }

        if(is_admin()){
            //loading translations
            load_plugin_textdomain('gravityformscapsulecrm', FALSE, '/gravityformscapsulecrm/languages' );

            add_filter("transient_update_plugins", array('GFCapsuleCRM', 'check_update'));
            add_filter("site_transient_update_plugins", array('GFCapsuleCRM', 'check_update'));

            add_action('install_plugins_pre_plugin-information', array('GFCapsuleCRM', 'display_changelog'));

            //creates a new Settings page on Gravity Forms' settings screen
            if(self::has_access("gravityforms_capsulecrm")){
                RGForms::add_settings_page("CapsuleCRM", array("GFCapsuleCRM", "settings_page"), self::get_base_url() . "/images/icon.png");
            }
        }

        //integrating with Members plugin
        if(function_exists('members_get_capabilities'))
            add_filter('members_get_capabilities', array("GFCapsuleCRM", "members_get_capabilities"));

        //creates the subnav left menu
        add_filter("gform_addon_navigation", array('GFCapsuleCRM', 'create_menu'));

        if(self::is_capsulecrm_page()){

            //enqueueing sack for AJAX requests
            wp_enqueue_script(array("sack"));

            //loading data lib
            require_once(self::get_base_path() . "/data.php");

            //loading upgrade lib
            if(!class_exists("RGCapsuleCRMUpgrade"))
                require_once("plugin-upgrade.php");

            //loading Gravity Forms tooltips
            require_once(GFCommon::get_base_path() . "/tooltips.php");
            add_filter('gform_tooltips', array('GFCapsuleCRM', 'tooltips'));

            //runs the setup when version changes
            self::setup();

         }
         else if(in_array(RG_CURRENT_PAGE, array("admin-ajax.php"))){

            //loading data class
            require_once(self::get_base_path() . "/data.php");

            add_action('wp_ajax_rg_update_config_active', array('GFCapsuleCRM', 'update_config_active'));
            add_action('wp_ajax_gf_select_capsulecrm_form', array('GFCapsuleCRM', 'select_capsulecrm_form'));

        }
        else{
             //handling post submission.
            add_action("gform_post_submission", array('GFCapsuleCRM', 'push'), 10, 2);
        }
    }

    public static function update_config_active(){
        check_ajax_referer('rg_update_config_active','rg_update_config_active');
        $id = $_POST["record_id"];
        $record = GFCapsuleCRMData::get_config($id);
        GFCapsuleCRMData::update_config($id, $record["form_id"], $_POST["is_active"], $record["meta"]);
    }

    //--------------   Automatic upgrade ---------------------------------------------------

    public static function flush_version_info(){
        if(!class_exists("RGCapsuleCRMUpgrade"))
            require_once("plugin-upgrade.php");

        RGCapsuleCRMUpgrade::set_version_info(false);
    }

    public static function plugin_row(){
        if(!self::is_gravityforms_supported()){
            $message = sprintf(__("Gravity Forms " . self::$min_gravityforms_version . " is required. Activate it now or %spurchase it today!%s"), "<a href='http://www.gravityforms.com'>", "</a>");
            RGCapsuleCRMUpgrade::display_plugin_message($message, true);
        }
        else{
            $version_info = RGCapsuleCRMUpgrade::get_version_info(self::$slug, self::get_key(), self::$version);

            if(!$version_info["is_valid_key"]){
                $new_version = version_compare(self::$version, $version_info["version"], '<') ? __('There is a new version of Gravity Forms CapsuleCRM Add-On available.', 'gravityformscapsulecrm') .' <a class="thickbox" title="Gravity Forms CapsuleCRM Add-On" href="plugin-install.php?tab=plugin-information&plugin=' . self::$slug . '&TB_iframe=true&width=640&height=808">'. sprintf(__('View version %s Details', 'gravityformscapsulecrm'), $version_info["version"]) . '</a>. ' : '';
                $message = $new_version . sprintf(__('%sRegister%s your copy of Gravity Forms to receive access to automatic upgrades and support. Need a license key? %sPurchase one now%s.', 'gravityformscapsulecrm'), '<a href="admin.php?page=gf_settings">', '</a>', '<a href="http://www.gravityforms.com">', '</a>') . '</div></td>';
                RGCapsuleCRMUpgrade::display_plugin_message($message);
            }
        }
    }

    //Displays current version details on Plugin's page
    public static function display_changelog(){
        if($_REQUEST["plugin"] != self::$slug)
            return;

        //loading upgrade lib
        if(!class_exists("RGCapsuleCRMUpgrade"))
            require_once("plugin-upgrade.php");

        RGCapsuleCRMUpgrade::display_changelog(self::$slug, self::get_key(), self::$version);
    }

    public static function check_update($update_plugins_option){
        if(!class_exists("RGCapsuleCRMUpgrade"))
            require_once("plugin-upgrade.php");

        return RGCapsuleCRMUpgrade::check_update(self::$path, self::$slug, self::$url, self::$slug, self::get_key(), self::$version, $update_plugins_option);
    }

    private static function get_key(){
        if(self::is_gravityforms_supported())
            return GFCommon::get_key();
        else
            return "";
    }
    //---------------------------------------------------------------------------------------

    //Returns true if the current page is an Record pages. Returns false if not
    private static function is_capsulecrm_page(){
        $current_page = trim(strtolower(rgget("page")));
        $capsulecrm_pages = array("gf_capsulecrm");

        return in_array($current_page, $capsulecrm_pages);
    }

    private static function setup(){
				// no database tables required. Just uses the GFMeta
        update_option("gf_capsulecrm_version", self::$version);
    }

    //Adds record tooltips to the list of tooltips
    public static function tooltips($tooltips){
        $capsulecrm_tooltips = array(
            "capsulecrm_contact_list" => "<h6>" . __("CapsuleCRM List", "gravityformscapsulecrm") . "</h6>" . __("Select the CapsuleCRM list you would like to add your contacts to.", "gravityformscapsulecrm"),
            "capsulecrm_gravity_form" => "<h6>" . __("Gravity Form", "gravityformscapsulecrm") . "</h6>" . __("Select the Gravity Form you would like to integrate with CapsuleCRM. Contacts generated by this form will be automatically added to your CapsuleCRM account.", "gravityformscapsulecrm"),
            "capsulecrm_welcome" => "<h6>" . __("Send Welcome Email", "gravityformscapsulecrm") . "</h6>" . __("When this option is enabled, users will receive an automatic welcome email from CapsuleCRM upon being added to your CapsuleCRM list.", "gravityformscapsulecrm"),
            "capsulecrm_map_fields" => "<h6>" . __("Map Fields", "gravityformscapsulecrm") . "</h6>" . __("Associate your CapsuleCRM merge variables to the appropriate Gravity Form fields by selecting.", "gravityformscapsulecrm"),
            "capsulecrm_optin_condition" => "<h6>" . __("Opt-In Condition", "gravityformscapsulecrm") . "</h6>" . __("When the opt-in condition is enabled, form submissions will only be exported to CapsuleCRM when the condition is met. When disabled all form submissions will be exported.", "gravityformscapsulecrm"),
            "capsulecrm_double_optin" => "<h6>" . __("Double Opt-In", "gravityformscapsulecrm") . "</h6>" . __("When the double opt-in option is enabled, CapsuleCRM will send a confirmation email to the user and will only add them to your CapsuleCRM list upon confirmation.", "gravityformscapsulecrm")

        );
        return array_merge($tooltips, $capsulecrm_tooltips);
    }

    //Creates CapsuleCRM left nav menu under Forms
    public static function create_menu($menus){

        // Adding submenu if user has access
        $permission = self::has_access("gravityforms_capsulecrm");
        if(!empty($permission))
				{
          //  $menus[] = array("name" => "gf_capsulecrm", "label" => __("CapsuleCRM", "gravityformscapsulecrm"), "callback" =>  array("GFCapsuleCRM", "capsulecrm_page"), "permission" => $permission);
				}
        return $menus;
    }

    public static function settings_page(){
				/* // REVISIT LATER - taken from MailChimp AddOn Plugin
        if(!class_exists("RGCapsuleCRMUpgrade"))
				*/ 
				if(rgpost("gf_capsulecrm_submit")){
            check_admin_referer("update", "gf_capsulecrm_update");
            $settings = array(
							"appname" => stripslashes($_POST["gf_capsulecrm_appname"]), 
							"apikey" => $_POST["gf_capsulecrm_apikey"]
						);
            update_option("gf_capsulecrm_settings", $settings);
        }
        else{
            $settings = get_option("gf_capsulecrm_settings");
        }
        //recordback for api key
        $recordback_image = "";
        $is_valid_apikey = false;
        if(!empty($settings["apikey"])){
            $is_valid_apikey = self::is_valid_login($settings["apikey"], $settings['appname']);
            $icon = $is_valid_apikey ? self::get_base_url() . "/images/tick.png" : self::get_base_url() . "/images/stop.png";
            $recordback_image = "<img src='{$icon}' />";
        }
        //if username is blank, hide username/password fields
        if(empty($settings["username"]) || $is_valid_apikey){
            $hidden_class = "hidden";
            $settings["username"] = "";
            $settings["password"] = "";
        }
        ?>
        <style>
            .valid_credentials{color:green;}
            .invalid_credentials{color:red;}
        </style>
        <form method="post" action="">
            <?php wp_nonce_field("update", "gf_capsulecrm_update") ?>
            <h3><?php _e("CapsuleCRM Account Information", "gravityformscapsulecrm") ?></h3>
            <p style="text-align: left;">
                <?php _e(sprintf("CapsuleCRM makes it easy to send email newsletters to your customers, manage your subscriber lists, and track campaign performance. Use Gravity Forms to collect customer information and automatically add them to your CapsuleCRM system as new opportunities. If you don't have a CapsuleCRM account, you can %ssign up for one here%s", "<a href='http://www.capsulecrm.com/' target='_blank'>" , "</a>"), "gravityformscapsulecrm") ?>
            </p>

            <table class="form-table">
		           <tr>
		               <th scope="row"><label for="gf_capsulecrm_appname"><?php _e("CapsuleCRM appName", "gravityformscapsulecrm"); ?></label> </th>
		               <td>
		                   <input type="text" style="text-align:right;" id="gf_capsulecrm_appname" name="gf_capsulecrm_appname" value="<?php echo esc_attr($settings["appname"]) ?>" size="30"/>.capsulecrm.com
		               </td>
		           </tr>
                <tr>
                    <th scope="row"><label for="gf_capsulecrm_apikey"><?php _e("CapsuleCRM API Key", "gravityformscapsulecrm"); ?></label> </th>
                    <td>
                        <input type="text" id="gf_capsulecrm_apikey" name="gf_capsulecrm_apikey" value="<?php echo esc_attr($settings["apikey"]) ?>" size="50"/>
                        <?php echo $recordback_image?>
                    </td>
                </tr>
                <tr>
                    <td colspan="2" ><input type="submit" name="gf_capsulecrm_submit" class="button-primary" value="<?php _e("Save Settings", "gravityformscapsulecrm") ?>" /></td>
                </tr>
            </table>
        </form>
        <?php
    }

    public static function capsulecrm_page(){
        $view = rgar($_GET,"view");
        if($view == "edit")
            self::edit_page($_GET["id"]);
        else
            self::list_page();
    }

    //Displays the capsulecrm records list page
    private static function list_page(){
        if(!self::is_gravityforms_supported()){
            die(__(sprintf("CapsuleCRM Add-On requires Gravity Forms %s. Upgrade automatically on the %sPlugin page%s.", self::$min_gravityforms_version, "<a href='plugins.php'>", "</a>"), "gravityformscapsulecrm"));
        }

        if(rgpost("action") == "delete"){
            check_admin_referer("list_action", "gf_capsulecrm_list");

            $id = absint($_POST["action_argument"]);
            GFCapsuleCRMData::delete_config($id);
            ?>
            <div class="updated fade" style="padding:6px"><?php _e("Record deleted.", "gravityformscapsulecrm") ?></div>
            <?php
        }
        else if (!empty($_POST["bulk_action"])){
            check_admin_referer("list_action", "gf_capsulecrm_list");
            $selected_configs = $_POST["record"];
            if(is_array($selected_configs)){
                foreach($selected_configs as $record_id)
                    GFCapsuleCRMData::delete_config($record_id);
            }
            ?>
            <div class="updated fade" style="padding:6px"><?php _e("Records deleted.", "gravityformscapsulecrm") ?></div>
            <?php
        }

        ?>
        <div class="wrap">
            <img alt="<?php _e("CapsuleCRM Records", "gravityformscapsulecrm") ?>" src="<?php echo self::get_base_url()?>/images/icon.png" style="float:left; margin:15px 7px 0 0;"/>
            <h2><?php _e("CapsuleCRM Records", "gravityformscapsulecrm"); ?>
            <a class="button add-new-h2" href="admin.php?page=gf_capsulecrm&view=edit&id=0"><?php _e("Add New", "gravityformscapsulecrm") ?></a>
            </h2>


            <form id="record_form" method="post">
                <?php wp_nonce_field('list_action', 'gf_capsulecrm_list') ?>
                <input type="hidden" id="action" name="action"/>
                <input type="hidden" id="action_argument" name="action_argument"/>

                <div class="tablenav">
                    <div class="alignleft actions" style="padding:8px 0 7px; 0">
                        <label class="hidden" for="bulk_action"><?php _e("Bulk action", "gravityformscapsulecrm") ?></label>
                        <select name="bulk_action" id="bulk_action">
                            <option value=''> <?php _e("Bulk action", "gravityformscapsulecrm") ?> </option>
                            <option value='delete'><?php _e("Delete", "gravityformscapsulecrm") ?></option>
                        </select>
                        <?php
                        echo '<input type="submit" class="button" value="' . __("Apply", "gravityformscapsulecrm") . '" onclick="if( jQuery(\'#bulk_action\').val() == \'delete\' && !confirm(\'' . __("Delete selected records? ", "gravityformscapsulecrm") . __("\'Cancel\' to stop, \'OK\' to delete.", "gravityformscapsulecrm") .'\')) { return false; } return true;"/>';
                        ?>
                    </div>
                </div>
                <table class="widefat fixed" cellspacing="0">
                    <thead>
                        <tr>
                            <th scope="col" id="cb" class="manage-column column-cb check-column" style=""><input type="checkbox" /></th>
                            <th scope="col" id="active" class="manage-column check-column"></th>
                            <th scope="col" class="manage-column"><?php _e("Form", "gravityformscapsulecrm") ?></th>
                            <th scope="col" class="manage-column"><?php _e("CapsuleCRM List", "gravityformscapsulecrm") ?></th>
                        </tr>
                    </thead>

                    <tfoot>
                        <tr>
                            <th scope="col" id="cb" class="manage-column column-cb check-column" style=""><input type="checkbox" /></th>
                            <th scope="col" id="active" class="manage-column check-column"></th>
                            <th scope="col" class="manage-column"><?php _e("Form", "gravityformscapsulecrm") ?></th>
                            <th scope="col" class="manage-column"><?php _e("CapsuleCRM List", "gravityformscapsulecrm") ?></th>
                        </tr>
                    </tfoot>

                    <tbody class="list:user user-list">
                        <?php

                        $settings = GFCapsuleCRMData::get_configs();
                        if(is_array($settings) && sizeof($settings) > 0){
                            foreach($settings as $setting){
                                ?>
                                <tr class='author-self status-inherit' valign="top">
                                    <th scope="row" class="check-column"><input type="checkbox" name="record[]" value="<?php echo $setting["id"] ?>"/></th>
                                    <td><img src="<?php echo self::get_base_url() ?>/images/active<?php echo intval($setting["is_active"]) ?>.png" alt="<?php echo $setting["is_active"] ? __("Active", "gravityformscapsulecrm") : __("Inactive", "gravityformscapsulecrm");?>" title="<?php echo $setting["is_active"] ? __("Active", "gravityformscapsulecrm") : __("Inactive", "gravityformscapsulecrm");?>" onclick="ToggleActive(this, <?php echo $setting['id'] ?>); " /></td>
                                    <td class="column-title">
                                        <a href="admin.php?page=gf_capsulecrm&view=edit&id=<?php echo $setting["id"] ?>" title="<?php _e("Edit", "gravityformscapsulecrm") ?>"><?php echo $setting["form_title"] ?></a>
                                        <div class="row-actions">
                                            <span class="edit">
                                            <a title="Edit this setting" href="admin.php?page=gf_capsulecrm&view=edit&id=<?php echo $setting["id"] ?>" title="<?php _e("Edit", "gravityformscapsulecrm") ?>"><?php _e("Edit", "gravityformscapsulecrm") ?></a>
                                            |
                                            </span>

                                            <span class="edit">
                                            <a title="<?php _e("Delete", "gravityformscapsulecrm") ?>" href="javascript: if(confirm('<?php _e("Delete this record? ", "gravityformscapsulecrm") ?> <?php _e("\'Cancel\' to stop, \'OK\' to delete.", "gravityformscapsulecrm") ?>')){ DeleteSetting(<?php echo $setting["id"] ?>);}"><?php _e("Delete", "gravityformscapsulecrm")?></a>

                                            </span>
                                        </div>
                                    </td>
                                    <td class="column-date"><?php echo $setting["meta"]["contact_list_name"] ?></td>
                                </tr>
                                <?php
                            }
                        }
                        else if(self::get_api()){
                            ?>
                            <tr>
                                <td colspan="4" style="padding:20px;">
                                    <?php _e(sprintf("You don't have any CapsuleCRM records configured. Let's go %screate one%s!", '<a href="admin.php?page=gf_capsulecrm&view=edit&id=0">', "</a>"), "gravityformscapsulecrm"); ?>
                                </td>
                            </tr>
                            <?php
                        }
                        else{
                            ?>
                            <tr>
                                <td colspan="4" style="padding:20px;">
                                    <?php _e(sprintf("To get started, please configure your %sCapsuleCRM Settings%s.", '<a href="admin.php?page=gf_settings&addon=CapsuleCRM">', "</a>"), "gravityformscapsulecrm"); ?>
                                </td>
                            </tr>
                            <?php
                        }
                        ?>
                    </tbody>
                </table>
            </form>
        </div>
        <script type="text/javascript">
            function DeleteSetting(id){
                jQuery("#action_argument").val(id);
                jQuery("#action").val("delete");
                jQuery("#record_form")[0].submit();
            }
            function ToggleActive(img, record_id){
                var is_active = img.src.indexOf("active1.png") >=0
                if(is_active){
                    img.src = img.src.replace("active1.png", "active0.png");
                    jQuery(img).attr('title','<?php _e("Inactive", "gravityformscapsulecrm") ?>').attr('alt', '<?php _e("Inactive", "gravityformscapsulecrm") ?>');
                }
                else{
                    img.src = img.src.replace("active0.png", "active1.png");
                    jQuery(img).attr('title','<?php _e("Active", "gravityformscapsulecrm") ?>').attr('alt', '<?php _e("Active", "gravityformscapsulecrm") ?>');
                }

                var mysack = new sack("<?php echo admin_url("admin-ajax.php")?>" );
                mysack.execute = 1;
                mysack.method = 'POST';
                mysack.setVar( "action", "rg_update_config_active" );
                mysack.setVar( "rg_update_config_active", "<?php echo wp_create_nonce("rg_update_config_active") ?>" );
                mysack.setVar( "record_id", record_id );
                mysack.setVar( "is_active", is_active ? 0 : 1 );
                mysack.encVar( "cookie", document.cookie, false );
                mysack.onError = function() { alert('<?php _e("Ajax error while updating record", "gravityformscapsulecrm" ) ?>' )};
                mysack.runAJAX();

                return true;
            }
        </script>
        <?php
    }

    private static function is_valid_login($apikey, $appname){
        require_once("api/Services/Capsule.php"); //CapsuleCrmAPI.class.php");
        $api = new Services_Capsule($appname, $apikey);
        $parties = $api->party->getList(0,1);
				return $parties;
    }

    private static function get_api(){
	    $settings = get_option("gf_capsulecrm_settings");
			return new Services_Capsule($settings['appname'], $settings['apikey']);
    }

    public static function add_permissions(){
        global $wp_roles;
        $wp_roles->add_cap("administrator", "gravityforms_capsulecrm");
        $wp_roles->add_cap("administrator", "gravityforms_capsulecrm_uninstall");
    }

    //Target of Member plugin filter. Provides the plugin with Gravity Forms lists of capabilities
    public static function members_get_capabilities( $caps ) {
        return array_merge($caps, array("gravityforms_capsulecrm", "gravityforms_capsulecrm_uninstall"));
    }

    public static function disable_capsulecrm(){
        delete_option("gf_capsulecrm_settings");
    }

		
		public static function get_field_selector($form_id, $field_name, $selected_field = null) {
			$form_fields = self::get_form_fields($form_id);
			$str = '<select id="'.$field_name.'" size="1" onchange=\'ChangeCapsuleCRMfield("'.$field_name.'");\'>';
			$str .= '<option value="">Choose</option>'."\n";
			foreach($form_fields as $_field) 
			{
				$str .= '<option value="'.$_field[0].'"';
				if($selected_field && $_field[0] == $selected_field) $str .= ' selected';
				$str .= '>'.$_field[1].'</option>'."\n";
			}
			$str .= '</select>'."\n";
			$str .= '<script> jQuery("#'.$field_name.'").val( form.'.$field_name.'); </script>'."\n";
			return $str;
		}
		
		public static function get_text_field($form_id, $field_name, $value = null) {
			$str = '<input style="width:100%" type="text" id="'.$field_name.'" value="" onblur=\'ChangeCapsuleCRMfield("'.$field_name.'");\'>';
			$str .= '<script> jQuery("#'.$field_name.'").val( form.'.$field_name.'); </script>'."\n";
			return $str;
		}
		
		public static function get_textarea_field($form_id, $field_name, $value = null) {
			$str = '<textarea rows="10" style="width:410px" id="'.$field_name.'" onblur=\'ChangeCapsuleCRMfield("'.$field_name.'");\'></textarea>';
			$str .= '<script> jQuery("#'.$field_name.'").val( form.'.$field_name.'); </script>'."\n";
			return $str;
		}
		
    public static function get_form_fields($form_id){
        $form = RGFormsModel::get_form_meta($form_id);
        $fields = array();

        //Adding default fields
        array_push($form["fields"],array("id" => "date_created" , "label" => __("Entry Date", "gfcapsulecrm")));
        array_push($form["fields"],array("id" => "ip" , "label" => __("User IP", "gfcapsulecrm")));
        array_push($form["fields"],array("id" => "source_url" , "label" => __("Source Url", "gfcapsulecrm")));

        if(is_array($form["fields"])){
            foreach($form["fields"] as $field){
                if(is_array(rgar($field, "inputs"))){

                    //If this is an address field, add full name to the list
                    if(RGFormsModel::get_input_type($field) == "address")
                        $fields[] =  array($field["id"], GFCommon::get_label($field) . " (" . __("Full" , "gfcapsulecrm") . ")");

                    foreach($field["inputs"] as $input)
                        $fields[] =  array($input["id"], GFCommon::get_label($field, $input["id"]));
                }
                else if(!rgar($field,"displayOnly")){
                    $fields[] =  array($field["id"], GFCommon::get_label($field));
                }
            }
        }
        return $fields;
    }


    public static function push($entry, $form){ //  was "export"
        $debug = false;
        //Login to CapsuleCRM
        $api = self::get_api();
        if(!$api) return;
        //loading data class
        $form_meta = RGFormsModel::get_form_meta($form['id']);
        if($debug) {
  				print '<pre>';
  				print '<h1>Form Meta</h1>';
  				print_r($form_meta);
  				print '<hr>';
  				print '<h1>Entry</h1>';
  				print_r($entry);
  				print '<hr>';
  				print '<h1>Organise the data</h1>';
  				print '<h1>Person</h1>';
			  }
				// 1. ACHTUNG! Translation of fields into the Person Record
				// See http://capsulecrm.com/help/page/javelin_api_party
				$person_fields = self::collate_person($entry, $form);
				// 2. Make an Organisation if necessary - http://developer.capsulecrm.com/v1/resources/parties/
				if($form['capsulecrm_organisation'] && $form['capsulecrm_organisation_name'])
				{
					$person_fields['organisationName'] = $entry[$form['capsulecrm_organisation_name']];
				}
				// 3. Save the Person (and get their party_id)
				if($debug) print_r($person_fields);
				$api->person->add($person_fields);
				$person_id = $api->person->get_new_id();
				// ---------------------------------------------------------------
				// 4. Person/Party Tags
				// http://capsulecrm.com/help/page/javelin_api_party_tags
				$person_tags = self::collate_tags($entry, $form, 'capsulecrm_person_tags');
				if(count($person_tags) > 0) 
				{
					foreach($person_tags as $tag_text) 
					{
						$api->party->tag->add($person_id, $tag_text);
					}
				}
				// debugging/testing
				$api->party->tag->add($person_id, 'via GravityForm');						
				// ---------------------------------------------------------------
				// 5. Add an opportunity - http://developer.capsulecrm.com/v1/resources/opportunities/
				if($form['capsulecrm_opportunity'] && $form['capsulecrm_opportunity_name'] != '')
				{
					$opportunity_fields = self::collate_opportunity($entry, $form, $person_id, $person_fields);
					if($form['capsulecrm_opportunity_track'] != '')
					{
					  $api->party->opportunity->add($person_id, $opportunity_fields, $form['capsulecrm_opportunity_track']);
				  }
				  else
				  {
				    $api->party->opportunity->add($person_id, $opportunity_fields);
				  }
					$opportunity_id = $api->party->opportunity->get_new_id();
					// 5.1 Add tags to the opportunity
					$opportunity_tags = self::collate_tags($entry, $form, 'capsulecrm_opportunity_tags');
					if(count($opportunity_tags) > 0) 
					{
						foreach($opportunity_tags as $tag_text) 
						{
							$api->opportunity->tag->add($opportunity_id, $tag_text);
						}
					}					
					$api->opportunity->tag->add($opportunity_id, 'GravityForm');					
					// 5.2 Add a task to the opportunity - No use a track instead.
					/*
					$api->opportunity->task->add($opportunity_id, array('description'=>'check new enquiry',
						'category'=>'Call', 'dueDateTime'=> date('Y-m-d').'T'.'15:30:00Z',
					));
					*/
					// 5.3 Add a note to the opportunity
				  $note_text = GFCommon::replace_variables( $form['capsulecrm_opportunity_note_text'], $form, $entry, false, false, false, 'text');
					$api->opportunity->history->addNote($opportunity_id, $note_text);
				}				
        if($debug) {
				  print '</pre>';
			  }
        //getting all active records
    }

		public static function collate_person($entry, $form) 
		{
			$name_bits = explode(' ', $entry[$form['capsulecrm_person_name']]);
			$lastName = array_pop($name_bits); 
			// debug // $lastName .= '-TESTING';
			return array(
				'firstName' => trim(join(' ', $name_bits)),
				'lastName' => $lastName,
				'contacts' => array(
					'email' => array('type' => 'work', 'emailAddress' => $entry[$form['capsulecrm_person_email']]),
					'phone' => array('type' => 'work', 'phoneNumber' => $entry[$form['capsulecrm_person_phone']]),
					'website' => array('type' => 'work', 'webService' => 'URL', 'webAddress' => $entry[$form['capsulecrm_person_url']]),
				),
				'about' => 'Created-automagically by MSAU-WP plugin at '.date('Y-m-d h:i:s'),
			);
		}
		
		public static function collate_opportunity($entry, $form, $person_id, $person_fields) 
		{
		  $opportunity_name = GFCommon::replace_variables( $form['capsulecrm_opportunity_name'], $form, $entry, false, false, false, 'text');
			$return = array(
				'name'=> $opportunity_name,
				'description' => '', 
				'partyId' => $person_id,
				'currency' => $form['capsulecrm_opportunity_currency'], 
				'value' => $form['capsulecrm_opportunity_value'], 
				'durationBasis' => 'FIXED', // ATTN: from Record
				'duration' => '1', // ATTN: from Record - Reqd if durationBasis is not FIXED
				'milestone' => 'Prospect', // ATTN: from Record
				// 'owner' => '', // ATTN: from Record - Currently authenticated user if not supplied
			);
			if($form['capsulecrm_opportunity_track'] != '')
			{
			  $return['trackId'] = $form['capsulecrm_opportunity_track'];
			}
			return $return;
		}
		

		public static function collate_tags($entry, $form, $field_name) 
		{
			if(!ereg(',', $form[$field_name])) return array('0' => $form[$field_name]);
			return explode(',', $form[$field_name]);
		}

    // public static function export_config($entry, $form, $record, $api){
    // }

    public static function uninstall(){

        //loading data lib
        require_once(self::get_base_path() . "/data.php");

        if(!GFCapsuleCRM::has_access("gravityforms_capsulecrm_uninstall"))
            die(__("You don't have adequate permission to uninstall CapsuleCRM Add-On.", "gravityformscapsulecrm"));

        //droping all tables
        // GFCapsuleCRMData::drop_tables();

        //removing options
        delete_option("gf_capsulecrm_settings");
        delete_option("gf_capsulecrm_version");

        //Deactivating plugin
        $plugin = "gravityformscapsulecrm/capsulecrm.php";
        deactivate_plugins($plugin);
        update_option('recently_activated', array($plugin => time()) + (array)get_option('recently_activated'));
    }

    private static function is_gravityforms_installed(){
        return class_exists("RGForms");
    }

    private static function is_gravityforms_supported(){
        if(class_exists("GFCommon")){
            $is_correct_version = version_compare(GFCommon::$version, self::$min_gravityforms_version, ">=");
            return $is_correct_version;
        }
        else{
            return false;
        }
    }

    protected static function has_access($required_permission){
        $has_members_plugin = function_exists('members_get_capabilities');
        $has_access = $has_members_plugin ? current_user_can($required_permission) : current_user_can("level_7");
        if($has_access)
            return $has_members_plugin ? $required_permission : "level_7";
        else
            return false;
    }

    //Returns the url of the plugin's root folder
    protected function get_base_url(){
        return plugins_url(null, __FILE__);
    }

    //Returns the physical path of the plugin's root folder
    protected function get_base_path(){
        $folder = basename(dirname(__FILE__));
        return WP_PLUGIN_DIR . "/" . $folder;
    }

}


function gfcapsulecrm_add_tooltips($gf_tooltips) 
{
	$gf_tooltips["capsulecrm_enable"] = "<h6>".__("Integrate form with CapsuleCRM", "gfcapsulecrm")."</h6>".__("Tick this box to integrate this form with CapsuleCRM. When this form is submitted successfulling the data will be added to CapsuleCRM.", "gfcapsulecrm");
	$gf_tooltips['capsulecrm_name_field'] = "<h6>".__("The Name Field", "gfcapsulecrm")."</h6>" . __("This Name will be split into firstName and lastName for CapsuleCRM. Single words become the lastName. <br><br>More than two words become:- <br>firstname = 'one two'<br>lastname = 'three'.", "gfcapsulecrm"); 
	$gf_tooltips['capsulecrm_email_field'] = "<h6>".__("The Email Field", "gfcapsulecrm")."</h6>" . __("This Email field will be used in CapsuleCRM as the \"Email (work)\" field", "gfcapsulecrm"); 
	$gf_tooltips['capsulecrm_phone_field'] = "<h6>".__("The Phone Field", "gfcapsulecrm")."</h6>" . __("This Phone field will be used in CapsuleCRM as the \"Phone (work)\" field", "gfcapsulecrm"); 
	// capsulecrm_person_tags
	// capsulecrm_person_url
	$gf_tooltips['capsulecrm_organisation_enable'] = "<h6>".__("The Organisation?", "gfcapsulecrm")."</h6>" . __("This Organisation field will be used in CapsuleCRM to add an organisation for the Person (if selected)", "gfcapsulecrm"); 
	$gf_tooltips['capsulecrm_organisation_name'] = "<h6>".__("The Organisation?", "gfcapsulecrm")."</h6>" . __("This Organisation field will be used in CapsuleCRM to add an organisation for the Person (if selected)", "gfcapsulecrm"); 
	// capsulecrm_organisation_tags
	// capsulecrm_opportunity_tags
	// ATTN: more to write
	// 
	return $gf_tooltips;
}
add_filter('gform_tooltips', 'gfcapsulecrm_add_tooltips');

function gfcapsulecrm_form_options($form_id) 
{
	// load the form for the field merge tag generators
  $form = RGFormsModel::get_form_meta($form_id);
  
?><h4>Create a new Person using fields</h4>
<table cellspacing="5" cellpadding="5">
<tr>
	<td align="right">Name field?</td>
	<td><?php echo GFCapsuleCRM::get_field_selector($form_id, 'capsulecrm_person_name');?> <?php gform_tooltip("capsulecrm_name_field"); ?></td>
</tr>
<tr>
	<td align="right">Email field?</td>
	<td><?php echo GFCapsuleCRM::get_field_selector($form_id, 'capsulecrm_person_email');?> <?php gform_tooltip("capsulecrm_email_field"); ?></td>
</tr>
<tr>
	<td align="right">Phone field?</td>
	<td><?php echo GFCapsuleCRM::get_field_selector($form_id, 'capsulecrm_person_phone');?> <?php gform_tooltip("capsulecrm_phone_field"); ?></td>
</tr>
<tr>
	<td align="right">URL field?</td>
	<td><?php echo GFCapsuleCRM::get_field_selector($form_id, 'capsulecrm_person_url');?> <?php gform_tooltip("capsulecrm_url_field"); ?></td>
</tr>
<tr>
	<td align="right">Person tags</td>
	<td><?php echo GFCapsuleCRM::get_text_field($form_id, 'capsulecrm_person_tags');?> <?php gform_tooltip("capsulecrm_person_tags"); ?></td>
</tr>
</table>
<h4><?php
echo '<label for="gform_capsulecrm_organisation">';
_e("Create an organisation?", "gfcapsulecrm");
?>&nbsp;&nbsp;<input type="checkbox" onclick="ToggleCapsuleCRMorganisation();" id="gform_capsulecrm_organisation" />
<?php gform_tooltip("capsulecrm_organisation_enable");
// echo addslashes($tooltip); 
echo '</label>';
?></h4>
<div id="gform_capsulecrm_organisation_container">
<table cellspacing="5" cellpadding="5">
<tr>
	<td align="right">Company field?</td>
	<td><?php echo GFCapsuleCRM::get_field_selector($form_id, 'capsulecrm_organisation_name');?> <?php gform_tooltip("capsulecrm_organisation_name"); ?></td>
</tr>
<tr>
	<td align="right">Organisation tags</td>
	<td><?php echo GFCapsuleCRM::get_text_field($form_id, 'capsulecrm_organisation_tags');?> <?php gform_tooltip("capsulecrm_organisation_tags"); ?></td>
</tr>
</table>
</div>
<h4><?php
echo '<label for="gform_capsulecrm_opportunity">';
_e("Create an opportunity?", "gfcapsulecrm");
?>&nbsp;&nbsp;<input type="checkbox" onclick="ToggleCapsuleCRMopportunity();" id="gform_capsulecrm_opportunity" />
<?php gform_tooltip("capsulecrm_opportunity_enable");
// echo addslashes($tooltip); 
echo '</label>';
?></h4>
<div id="gform_capsulecrm_opportunity_container">
<table cellspacing="5" cellpadding="5">
<tr>
	<td colspan="1" align="left">Opportunity name</td>
	<td><?php  GFCommon::insert_variables($form['fields'], "capsulecrm_opportunity_name", true); ?></td>
</tr>
<tr>
	<td colspan="2"><?php 
	echo ''.GFCapsuleCRM::get_text_field($form_id, 'capsulecrm_opportunity_name');
	gform_tooltip("capsulecrm_opportunity_name"); 
	?></td>
</tr>
<tr>
	<td align="right">Opportunity tags</td>
	<td><?php echo GFCapsuleCRM::get_text_field($form_id, 'capsulecrm_opportunity_tags');?> <?php gform_tooltip("capsulecrm_opportunity_tags"); ?></td>
</tr>
<tr>
	<td align="right">Opportunity Currency, e.g. USD</td>
	<td><?php echo GFCapsuleCRM::get_text_field($form_id, 'capsulecrm_opportunity_currency');?> <?php gform_tooltip("capsulecrm_opportunity_currency"); ?></td>
</tr>
<tr>
	<td align="right">Opportunity Value</td>
	<td><?php echo GFCapsuleCRM::get_text_field($form_id, 'capsulecrm_opportunity_value');?> <?php gform_tooltip("capsulecrm_opportunity_value"); ?></td>
</tr>
<tr>
	<td align="right">Opportunity Track ID</td>
	<td><?php echo GFCapsuleCRM::get_text_field($form_id, 'capsulecrm_opportunity_track');?> <?php gform_tooltip("capsulecrm_opportunity_track"); ?></td>
</tr>
</table>
	<h4><?php
	echo '<label for="gform_capsulecrm_opportunity_note">';
	_e("Create a note on the opportunity?", "gfcapsulecrm");
	?>&nbsp;&nbsp;<input type="checkbox" onclick="ToggleCapsuleCRMopportunitynote();" id="gform_capsulecrm_opportunity_note" />
	<?php gform_tooltip("capsulecrm_opportunity_note_enable");
	// echo addslashes($tooltip); 
	echo '</label>';
	?></h4>
	<div id="gform_capsulecrm_opportunity_note_container" style="display:none;">
  <?php GFCommon::insert_variables($form['fields'], "capsulecrm_opportunity_note_text",false); ?>
	<table cellspacing="5" cellpadding="5">
	<tr>
		<td colspan="2"><?php echo GFCapsuleCRM::get_textarea_field($form_id, 'capsulecrm_opportunity_note_text');?> <?php gform_tooltip("capsulecrm_opportunity_note_text"); ?></td>
	</tr>
	</table>
</div>
<?php
}

function gfcapsulecrm_form_settings($position, $form_id = '') 
{
  if($position != '800') return;
  if($_GET['id']) {
	  echo '<li><input type="checkbox" onclick="ToggleCapsuleCRM();" id="gform_capsulecrm" /> ';
	  echo '<label for="gform_enable_capsulecrm" id="gform_enable_capsulecrm_label">';
	  _e("Enable CapsuleCRM integration", "gfcapsulecrm");
		gform_tooltip("capsulecrm_enable");
	  // echo addslashes($tooltip); 
	  echo '</label></li>';
    echo '<li id="gform_capsulecrm_container" style="display:none">';
		gfcapsulecrm_form_options($_GET['id']);
		echo '</li>';
?>
<script>
function ToggleCapsuleCRM(isInit)
{
    var speed = isInit ? "" : "slow";
    if(jQuery("#gform_capsulecrm").is(":checked")) 
        jQuery("#gform_capsulecrm_container").show(speed);		
    else
        jQuery("#gform_capsulecrm_container").hide(speed);
		form.capsulecrm_enabled = jQuery("#gform_capsulecrm").is(":checked");
}
function ToggleCapsuleCRMorganisation(isInit)
{
    var speed = isInit ? "" : "slow";
    if(jQuery("#gform_capsulecrm_organisation").is(":checked")) 
        jQuery("#gform_capsulecrm_organisation_container").show(speed);		
    else
        jQuery("#gform_capsulecrm_organisation_container").hide(speed);
		form.capsulecrm_organisation = jQuery("#gform_capsulecrm_organisation").is(":checked");
}
function ToggleCapsuleCRMopportunity(isInit)
{
    var speed = isInit ? "" : "slow";
    if(jQuery("#gform_capsulecrm_opportunity").is(":checked")) 
        jQuery("#gform_capsulecrm_opportunity_container").show(speed);		
    else
        jQuery("#gform_capsulecrm_opportunity_container").hide(speed);
		form.capsulecrm_opportunity = jQuery("#gform_capsulecrm_opportunity").is(":checked");
}
function ToggleCapsuleCRMopportunitynote(isInit)
{
    var speed = isInit ? "" : "slow";
    if(jQuery("#gform_capsulecrm_opportunity_note").is(":checked")) 
        jQuery("#gform_capsulecrm_opportunity_note_container").show(speed);		
    else
        jQuery("#gform_capsulecrm_opportunity_note_container").hide(speed);
		form.capsulecrm_opportunity_note = jQuery("#gform_capsulecrm_opportunity_note").is(":checked");
}
function ChangeCapsuleCRMfield(field_name) 
{
	//alert(jQuery("#"+field_name).val());
	eval('form.'+field_name+' = jQuery("#"+field_name).val();');
	//alert(form.capsulecrm_person_email);
}
jQuery("#gform_capsulecrm").attr("checked", form.capsulecrm_enabled ? true : false);
jQuery("#gform_capsulecrm_organisation").attr("checked", form.capsulecrm_organisation ? true : false);
jQuery("#gform_capsulecrm_opportunity").attr("checked", form.capsulecrm_opportunity ? true : false);
jQuery("#gform_capsulecrm_opportunity_note").attr("checked", form.capsulecrm_opportunity_note ? true : false);
ToggleCapsuleCRM(true);
ToggleCapsuleCRMorganisation(true);
ToggleCapsuleCRMopportunity(true);
ToggleCapsuleCRMopportunitynote(true);
</script><?php
  }
  return;
}
add_action("gform_advanced_settings", 'gfcapsulecrm_form_settings', 10);

